﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Strom
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void showItems (MyNode branch,
                                DirectoryInfo dir,
                                int level)
        {
            try
            {
                DirectoryInfo[] list = dir.GetDirectories();
                branch.ready = true;
                if (list != null)
                    foreach (DirectoryInfo entry in list)
                        showBranch (branch.Nodes, entry, level - 1);
            }
            catch (Exception ex)
            {
                TreeNode node = new TreeNode();
                node.Text = ex.Message;
                node.ForeColor = Color.Red;
                branch.Nodes.Add (node);
            }
        }

        private void showBranch (TreeNodeCollection above,
                                 DirectoryInfo dir,
                                 int level)
        {
            MyNode branch = new MyNode();
            branch.Text = dir.Name;
            branch.path = dir.FullName;
            branch.ToolTipText = branch.path;
            above.Add(branch);

            if (level > 1)
                showItems (branch, dir, level-1);
        }

        private void testButton_Click(object sender, EventArgs e)
        {
            // using System.IO;
            treeView1.ShowNodeToolTips = true;

            DriveInfo[] disks = DriveInfo.GetDrives();
            foreach (DriveInfo d in disks)
               showBranch (treeView1.Nodes, d.RootDirectory, 2);

            /*
            string[] s = { "abc", "def", "klm", "xyz" };
            dataGridView1.Rows.Add(s);
            dataGridView1.Columns.Add("a", "aaa");
            */
        }

        private void treeView1_BeforeExpand(object sender, TreeViewCancelEventArgs e)
        {
            MyNode node = e.Node as MyNode;
            if (node != null)
            {
                if (node.ready)
                {
                    foreach (TreeNode n in node.Nodes)
                        if (n is MyNode)
                        {
                            MyNode sub = n as MyNode;
                            if (! sub.ready)
                            {
                                DirectoryInfo dir = new DirectoryInfo(sub.path);
                                showItems (sub, dir, 1);
                            }
                        }
                }
                else
                {
                    // DirectoryInfo dir = new DirectoryInfo(node.path);
                    // showItems (node, dir, 2);
                }
            }
        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            listView1.Items.Clear();
            listView1.ShowItemToolTips = true;
            toolStripStatusLabel1.Text = "";
            MyNode node = e.Node as MyNode;
            if (node != null)
            {
                DirectoryInfo dir = new DirectoryInfo(node.path);
                try
                {
                    FileInfo[] files = dir.GetFiles();
                    if (files != null)
                        foreach (FileInfo f in files)
                        {
                            ListViewItem item = new ListViewItem();
                            item.Text = f.Name;
                            item.ToolTipText = f.FullName;
                            item.SubItems.Add ("" + f.Length);
                            item.SubItems.Add ("" + f.LastWriteTime);
                            item.Tag = f.FullName;
                            listView1.Items.Add(item);
                        }
                }
                catch (Exception ex)
                {
                }
            }

        }

        private string readFile(string path)
        {
            string result = "";
            StringBuilder b = new StringBuilder();
            StreamReader r = new StreamReader(path);
            int n = r.Read();
            while (n != -1)
            {
                char c = (char)n;
                // result += c;
                b.Append(c);
                n = r.Read();
            }

            // return result;
            return b.ToString();
        }

        private void readData (string path)
        {
            StreamReader r = new StreamReader(path);
            List<String> list = new List<String>();
            StringBuilder b = new StringBuilder();
            int n = r.Read();
            while (n != -1)
            {
                char c = (char)n;
                if (char.IsLetterOrDigit(c))
                {
                    b.Append(c);
                }
                else if (c == '\r')
                {
                }
                else if (c == '\n')
                {
                    if (b.Length != 0)
                        list.Add(b.ToString());

                    string[] items = list.ToArray();

                    while (dataGridView1.ColumnCount < items.Length)
                        dataGridView1.Columns.Add("", "");

                    dataGridView1.Rows.Add(items);

                    list = new List<string>();
                    b = new StringBuilder();
                }
                else
                {
                    if (b.Length != 0)
                        list.Add(b.ToString());
                    list.Add("" + c);
                    b = new StringBuilder();
                }

                n = r.Read();
            }

            {
                if (b.Length != 0)
                    list.Add(b.ToString());

                string[] items = list.ToArray();

                while (dataGridView1.ColumnCount < items.Length)
                    dataGridView1.Columns.Add("", "");

                dataGridView1.Rows.Add(items);
            }
        }

        private void listView1_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            pictureBox1.Image = null;
            richTextBox1.Text = "";

            if (e.IsSelected)
            {
                ListViewItem item = e.Item;
                string path = item.Tag as string;
                toolStripStatusLabel1.Text = path;
                string ext = Path.GetExtension (path);
                ext = ext.ToLower ();
                switch (ext)
                {
                    case ".pas":
                        readData (path);
                        controlTab1.SelectedTab = dataTab;
                        break;

                    case ".c":
                    case ".cpp":
                        richTextBox1.Text = readFile (path);
                        controlTab1.SelectedTab = textTab;
                        break;

                    case ".txt":
                        richTextBox1.LoadFile(path, RichTextBoxStreamType.PlainText);
                        controlTab1.SelectedTab = textTab;
                        break;

                    case ".rtf":
                        richTextBox1.LoadFile (path);
                        controlTab1.SelectedTab = textTab;
                        break;

                    case ".htm":
                    case ".html":
                        webBrowser1.Url = new Uri ("file://" + path);
                        controlTab1.SelectedTab = htmlTab;
                        break;

                    case ".jpg":
                    case ".jpeg":
                    case ".bmp":
                        pictureBox1.Image = new Bitmap (path);
                        pictureBox1.SizeMode = PictureBoxSizeMode.Zoom;
                        controlTab1.SelectedTab = imageTab;
                        break;
               }
            }
        }

        private void tabPage2_Click(object sender, EventArgs e)
        {

        }
    }
}
